#!/bin/bash
# BlueXFER Installer v0.2
# Alex N. Ibrado <alex@ibrado.com>
# Released under GPL 2

_check_reqt () 
{
	local prompt=$1
	local cmd=$2
	local req=$3

	loc=`which $2 2>/dev/null`
	# Remove dashes
	local var=${cmd//-/}
	if [ "$loc" = "" ]; then
		loc="Not found"
		let "HAS_$var=0"
	else
		let "HAS_$var=1"
		eval PATH_$var=$loc
	fi
	echo -e "$prompt: $loc"
	if [ "$req" ] && [ "$loc" = "Not found" ]; then
		echo
		echo "Please install $prompt first."
		exit
	fi
}

_get_ans () {
	local prompt=$1;
	local def=$2;
	local keepcase=$3;

	if [ "$def" ]; then
		default=" ($def)"
	fi
	echo
	echo -n "$prompt$default "
	read -e ANS
	if [ "$ANS" = "" ]; then
		ANS=$def
	fi
	if [ "$keepcase" = "" ]; then
		ANS=`echo $ANS | tr [A-Z] [a-z]`
	fi
}

_copy_backup () {
	local src=$1;
	local destdir=$2;
	local destname=$3;

	if [ "$destname" = "" ]; then 
		destname=$src;
	fi

	if [ -e "$destdir/$destname" ]; then
		local COUNT=1
		while [ -e "$destdir/$destname.old.$COUNT" ]; do
			let "COUNT=COUNT+1"
		done
		cp -f "$src" "$destdir/$destname.old.$COUNT" 
		echo "> Renamed existing $destdir/$destname to $destname.old.$COUNT"
	fi
	cp -f "$src" "$destdir/$destname" 
	echo "Copied $destname to $destdir"
}


#######################################
### Main script
#######################################

echo BlueXFER Installer v0.2
echo "Alex Ibrado <alex@ibrado.com>"

## Requirements
echo
echo "** Checking for requirements:"
echo

_check_reqt Perl perl 1
_check_reqt rfcomm rfcomm 1
_check_reqt sdptool sdptool 1
_check_reqt hcitool hcitool 1
_check_reqt sdpd sdpd 1

SDP_ON=`ps ax | grep sdpd | grep -v grep`;
if [ "$SDP_ON" = "" ]; then
	echo "> WARNING: sdpd is not running\n"
fi

_check_reqt hcid hcid 1

HCI_ON=`ps ax | grep hcid | grep -v grep`;
if [ "$HCI_ON" = "" ]; then
	echo "> WARNING: hcid is not running\n"
fi

_check_reqt mktemp mktemp 1
_check_reqt sudo sudo 1
_check_reqt ussp-push ussp-push

if [ $HAS_ussppush -eq 0 ]; then
	PATH_ussppush="ussp-push";
fi

ldd "$PATH_ussppush" | cut -f2 -d">" | cut -f2 -d " " | while read USSP_LIB; do
	if [ ! -e "$USSP_LIB" ]; then
		echo "* ussp-push requires $USSP_LIB"
		echo "* Please install the OpenOBEX libraries."
		exit
	else
		echo "> $USSP_LIB OK"
	fi
done

_check_reqt wget wget
_check_reqt Xdialog Xdialog
_check_reqt kdialog kdialog

echo

if [ $HAS_wget -eq 0 ]; then 
	echo "- Cannot automatically download files"
fi

if [ $HAS_Xdialog -eq 0 ]; then 
	if [ $HAS_kdialog -eq 1 ]; then
		echo "- Will use kdialog instead of Xdialog"
	else
		# Basically impossible unless deleted or no KDE(!) ... kdialog is part of kdebase
		echo "* Xdialog and kdialog are both not available."
		echo "* Cannot proceed -- please install at least one of them."
		exit
	fi
	echo "- Cannot display progress indicators for scans, downloads, and transfers"
fi

## Paths
echo
echo "** Configuring paths:"
EXEC_DIR="$HOME/bin";
while [ 1 ]; do
	_get_ans "Where do I place the executables?" "$EXEC_DIR" 1
	EXEC_DIR=$ANS

	if [ ! -e "$EXEC_DIR" ]; then
		_get_ans "$EXEC_DIR does not exist. Create it?" y
		if [ "$ANS" = "y" ]; then
			mkdir -p "$EXEC_DIR" 2>/dev/null
			if [ $? -gt 0 ]; then
				echo "* Error creating $EXEC_DIR."
				echo "* You may have insufficient privileges."
			fi
		fi
		echo
	else
		touch "$EXEC_DIR/.bluexfer-install-write-test" 2>/dev/null
		if [ $? -gt 0 ]; then
			echo "* Cannot write to $EXEC_DIR."
			echo "* You may have insufficient privileges."
		else
			rm -f "$EXEC_DIR/.bluexfer-install-write-test"
			break
		fi
	fi
done

HAS_EXEC_IN_PATH=`echo $PATH | grep "$EXEC_DIR"`
if [ "$HAS_EXEC_IN_PATH" = "" ]; then
	_get_ans "Would you like to add $EXEC_DIR to your PATH?" y
	if [ "$ANS" = "y" ]; then
		if [ -e "$EXEC_DIR" ]; then
			echo "export PATH=\$PATH:$EXEC_DIR" >> "$HOME/.bash_profile"
			export PATH="$PATH:$EXEC_DIR"
			echo "> Added 'export PATH=\$PATH:$EXEC_DIR' to $HOME/.bash_profile"
			echo "> You may need to logout and back in for the changes to take effect globally."
		else
			echo "> Not adding $HOME/bin to PATH -- directory does not exist."
		fi
	else
		echo "> Not adding $HOME/bin to PATH -- you must do this manually."
	fi
fi


LOCAL_KDE=`dirname \`kde-config --localprefix\`dummy`
if [ "$LOCAL_KDE" = "" ]; then
	LOCAL_KDE="$HOME/.kde"
fi

_get_ans "What is your local KDE directory?" "$LOCAL_KDE" 1
LOCAL_KDE=$ANS

KONQ_SERV=`find "$LOCAL_KDE" -name "servicemenus" | grep konqueror`
_get_ans "Where should I put the service menu?" "$KONQ_SERV" 1
SERV_DIR=$ANS

KDE_GLOBS=`find "$LOCAL_KDE" -name "kdeglobals" | grep config`
THEME=`cat $KDE_GLOBS | grep Theme | cut -f2 -d=`
THEME_ICONS=`find "$LOCAL_KDE" -name "$THEME" | grep icons`
THEME_DIR="$THEME_ICONS/16x16/devices";

_get_ans "Where should I put the icon?" "$THEME_DIR" 1
ICON_DIR=$ANS


## The variables in the script
echo
echo
echo "** Configuring script variables and behavior:"

_get_ans "Which /dev/rfcomm device should BlueXFER use?" 0
RFCOMM_DEV=$ANS

USE_KD=0
USE_XD_PROG=0
if [ $HAS_kdialog -eq 1 ] && [ $HAS_Xdialog -eq 1 ]; then
	_get_ans "Would you like to use kdialog for most dialogs?" y
	if [ "$ANS" = "y" ]; then
		USE_KD=1
	fi
	if [ $HAS_Xdialog -eq 1 ] && [ $USE_KD -eq 1 ]; then
		_get_ans "Would you like to use Xdialog for progress indicators?" y
		if [ "$ANS" = "y" ]; then
			USE_XD_PROG=1
		else
			echo "> No progress indicators will be displayed"
		fi
	fi
elif [ $HAS_kdialog -eq 1 ]; then
	USE_KD=1
fi

_get_ans "Where would you like BlueXFER to store its info?" "\$ENV{HOME}/.bluexfer-ai" 1
STATE_FILE=$ANS


FIX_SUDO=0
_get_ans "Would you like to check if your sdptool requires sudo access?" y
if [ "$ANS" = "y" ]; then
	_get_ans "> Please turn on a Bluetooth device, wait a few seconds, then press" "Enter"

	RETRIES=5
	for ((i=1; i<=$RETRIES ; i++)) do 
		echo ">> Scanning for Bluetooth devices..."
		TESTDEV=`hcitool inq | grep class | cut -f2`

		if [ "$TESTDEV" != "" ]; then
			echo ">> Attempting to browse $TESTDEV"
			TESTSDP=`sdptool browse $TESTDEV 2>/dev/null | grep Service | head -1`
			if [ "$TESTSDP" != "" ]; then
				echo ">>> sudo is not required for sdptool on your system"
				break
			fi
		fi

		if [ $i -eq $RETRIES ]; then
			if [ "$TESTDEV" = "" ]; then
				echo ">>> Could not find any Bluetooth devices"
				echo ">>> Assuming sudo is not required"
			else
				echo ">>> sudo is required for sdptool on your system"
				FIX_SUDO=1
			fi
		else
			echo ">> Retrying in 5 seconds ($i/$RETRIES)..."
			sleep 5
		fi
	done
fi

FIX_CONFIRM=0
_get_ans "In case of multiple files, would you like confirmation for each successful send?" n
if [ "$ANS" = "n" ]; then
	FIX_CONFIRM=1
fi

GNOME_SCRIPTS=""
if [ -e "$HOME/.gnome2" ]; then
	echo
	echo
	echo "** Optional installation:"

	_get_ans "Would you like to install the GNOME2/Nautilus Scripts Folder files?" n
	if [ "$ANS" = "y" ]; then
		_get_ans "Enter Nautilus Scripts Folder location:" "$HOME/.gnome2/nautilus-scripts" 1
		GNOME_SCRIPTS=$ANS
	fi
fi


## Fix the script

FIX_PERL=0
if [ "$PATH_perl" != "/usr/bin/perl" ]; then
	FIX_PERL=1
fi

echo

IFS="^"
cat bluexfer | while read -r LINE; do
	if [ $FIX_PERL -eq 1 ] && [ "$LINE" = "#!/usr/bin/perl" ]; then
		echo -E "#!$PATH_perl"
    elif [ "$LINE" = "\$RFCOMM=\"0\";" ]; then
		echo -E "\$RFCOMM=\"$RFCOMM_DEV\";"
    elif [ "$LINE" = "\$USE_KDIALOG=0;" ]; then
		echo -E "\$USE_KDIALOG=$USE_KD;"
    elif [ "$LINE" = "\$USE_XDIALOG_PROGRESS=1;" ]; then
		echo -E "\$USE_XDIALOG_PROGRESS=$USE_XD_PROG;"
    elif [ "$LINE" = "\$STATE_FILE=\"\$ENV{HOME}/.bluexfer-ai\";" ]; then
		echo -E "\$STATE_FILE=\"$STATE_FILE\";"
	elif [ $FIX_SUDO -eq 1 ] && [ "$LINE" = "\$SDPTOOL=\"sdptool\";" ]; then
		echo -E "\$SDPTOOL=\"sudo sdptool\";"
	elif [ $FIX_CONFIRM -eq 1 ] && [ "$LINE" = "\$CONFIRM_MULTI=1;" ]; then
		echo -E "\$CONFIRM_MULTI=0;"
	else
		echo -E "$LINE"
	fi
done > bluexfer.custom

unset IFS

## Copy the files over

echo
echo "** Copying files:"
echo

_copy_backup "bluexfer.custom" "$EXEC_DIR" "bluexfer"
chmod +x "$EXEC_DIR/bluexfer"
rm -f bluexfer.custom

# No need to backup the following
cp -f "bluexfer.desktop" $SERV_DIR
echo "Copied bluexfer.desktop to $SERV_DIR"

cp -f "bluexfer-ai.png" $ICON_DIR
echo "Copied bluexfer-ai.png to $ICON_DIR"

if [ $HAS_ussppush -eq 0 ]; then
	cp -f ussp-push "$EXEC_DIR"
	chmod +x "$EXEC_DIR/ussp-push"
	echo "Copied ussp-push to $EXEC_DIR"
fi

if [ "$GNOME_SCRIPTS" != "" ]; then
	mkdir -p "$GNOME_SCRIPTS" 2>/dev/null
	cp -f "nautilus/Send via Bluetooth" "$GNOME_SCRIPTS"
	chmod +x "$GNOME_SCRIPTS/Send via Bluetooth"
	echo "Copied 'nautilus/Send via Bluetooth' to $GNOME_SCRIPTS"

	cp -f "nautilus/Send via Bluetooth to..." "$GNOME_SCRIPTS"
	chmod +x "$GNOME_SCRIPTS/Send via Bluetooth to..."
	echo "Copied 'nautilus/Send via Bluetooth to...' to $GNOME_SCRIPTS"
fi


## sudo
echo
_get_ans "Would you like to configure sudo access now?" n

if [ $FIX_SUDO -eq 1 ]; then
	SD_SUDO=", $PATH_sdptool"
else
	SD_SUDO=""
fi
	

echo
echo "Please make sure that the following line (or equivalent) exists in /etc/sudoers:"
echo
echo "$USER   ALL=(root) NOPASSWD: $PATH_rfcomm $SD_SUDO"

if [ "$ANS" = "y" ]; then
	echo "I will now run visudo."
	echo -n "Please enter root password: "
	su -c "visudo" 2>/dev/null
fi

echo
echo
echo '*** Configuration complete! ***'
echo
echo "Right-clicking on files within Konqueror will give you"
echo "the Bluetooth service menus."

if [ "$GNOME_SCRIPTS" != "" ]; then
	echo
	echo "In Nautilus, the Bluetooth service menus are accessible"
	echo "under the 'Scripts' context menu entry."
fi

echo
echo 'Please send comments/suggestions to alex@ibrado.com'
echo ' with "bluexfer" in the subject line.'
echo
echo 'Enjoy!'
echo



